/*
 * Copyright © 2021-2022 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3SOURCE_H
#define ENATSC3SOURCE_H

typedef struct _ENATSC3Source *ENATSC3Source;

// Callback function for changes in the list of PLPs carrying LLS. The list of 
// PLPs is indicated by a 64-bit mask (plpIdMask).
typedef void (*ENATSC3SourceLLSPLPsCallbackFunction)(uint64_t plpIdMask,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber,
    void *pUserData);

// Callback function for a baseband packet.
typedef void (*ENATSC3SourceBasebandPacketCallbackFunction)(uint32_t plpId, 
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    int32_t fError, int32_t fDiscontinuity, 
    const ENATCS3L1Time *pL1Time, const uint8_t *pPayload, 
    int32_t nPayloadBytes, void *pUserData);

// Callback function for an ALP packet
typedef void (*ENATSC3SourceALPPacketCallbackFunction)(uint32_t plpId, 
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    int32_t fError, int32_t fDiscontinuity,
    const ENATCS3L1Time *pL1Time, const uint8_t *pPayload,
    int32_t nPayloadBytes, void *pUserData);

// ATSC3Sources are created by device-specific Open functions which return an
// ENATSC3Source handle. Methods are called via the handle.
//
typedef ENStatus (*ENATSC3SourceStart)(ENATSC3Source hSource,
    int32_t *pFRunning);

typedef ENStatus (*ENATSC3SourceClose)(ENATSC3Source hSource);

typedef ENStatus (*ENATSC3SourceSetPLPIds)(ENATSC3Source hSource, 
    uint64_t plpIdMask);

typedef ENStatus (*ENATSC3SourceRegisterCallbacks)(ENATSC3Source hSource,
    ENATSC3SourceLLSPLPsCallbackFunction LLSPLPsCallback,
    ENATSC3SourceBasebandPacketCallbackFunction BasebandPacketCallback,
    ENATSC3SourceALPPacketCallbackFunction ALPPacketCallback,
    void *pUserData);

typedef struct _ENATSC3Source
{
    ENATSC3SourceStart Start;
    ENATSC3SourceClose Close;
    ENATSC3SourceSetPLPIds SetPLPIds;
    ENATSC3SourceRegisterCallbacks RegisterCallbacks;
    char strModel[32];
    char strSerial[32];
} *ENATSC3Source;

#endif
