/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3DASHFEEDER_H
#define ENATSC3DASHFEEDER_H

typedef struct _ENATSC3DASHFeeder *ENATSC3DASHFeederHandle_t;

typedef ENStatus (*ENATSC3DASHFeederMediaFileBeginCallback)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t tsi, 
    uint32_t toi, uint8_t codepoint, const char *pId, 
    ENATSC3ROUTESessionIdType idType, const char *pURL, 
    const uint8_t *pPayload, int32_t payloadLength, int32_t fError, 
    void *pUserData);

typedef void (*ENATSC3DASHFeederMediaFileEndCallback)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t tsi, 
    uint32_t toi, const char *pURL, int64_t objectSize, int64_t objectPosition, 
    void *pUserData);

typedef void (*ENATSC3DASHFeederMediaFileDataCallback)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, 
    uint32_t tsi, uint32_t toi, const char *pURL, const uint8_t *pPayload, 
    int32_t payloadLength, int64_t objectPosition, int64_t objectSize, 
    int32_t fError, void *pUserData);

typedef void (*ENATSC3DASHFeederMediaFileAbortCallback)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t tsi, 
    uint32_t toi, const char *pURL, int64_t objectPosition, int64_t objectSize, 
    void *pUserData);

typedef void (*ENATSC3DASHFeederLCTChannelRemovedCallback)(
    struct timespec *pPacketTimeUTC, uint64_t packetNumber, uint32_t tsi, 
    void *pUserData);

ENStatus ENATSC3DASHFeederAllocate(
    ENATSC3ROUTESessionHandle_t hROUTESession, const char *pDebugName, 
    ENATSC3DASHFeederHandle_t *phENATSC3DASHFeeder);

ENStatus ENATSC3DASHFeederFree(
    ENATSC3DASHFeederHandle_t hENATSC3DASHFeeder);

ENStatus ENATSC3DASHFeederRegisterCallbacks(
    ENATSC3DASHFeederHandle_t hENATSC3DASHFeeder,
    ENATSC3DASHFeederMediaFileBeginCallback MediaFileBeginCallback,
    ENATSC3DASHFeederMediaFileEndCallback MediaFileEndCallback,
    ENATSC3DASHFeederMediaFileAbortCallback MediaFileAbortCallback,
    ENATSC3DASHFeederMediaFileDataCallback MediaFileDataCallback,
    ENATSC3DASHFeederLCTChannelRemovedCallback LCTChannelRemovedCallback,
    void *pCallbackUserData);

ENStatus ENATSC3DASHFeederSetVerbosity(
    ENATSC3DASHFeederHandle_t hENATSC3DASHFeeder, ENVerbosity verbosity);

#endif
