/*
 * Copyright © 2020-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3ESG_SGDD_H
#define ATSC3ESG_SGDD_H

typedef struct
{
    EnXMLParserNodeBase base;

    int64_t startTime;
    int64_t endTime;
} ATSC3ESGSGDDTimeGroupingCriteriaNode;

#define ATSC3ESGSGDDTimeGroupingCriteriaNodeAttribute_startTime         0x0001
#define ATSC3ESGSGDDTimeGroupingCriteriaNodeAttribute_endTime           0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    ATSC3ESGSGDDTimeGroupingCriteriaNode timeGroupingCriteria;
} ATSC3ESGSGDDGroupingCriteriaNode;

#define ATSC3ESGSGDDGroupingCriteriaChildNode_GroupingCriteria          0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t ipAddressDataOffset;
    int32_t port;
    int32_t transmissionSessionID;
} ATSC3ESGSGDDTransportNode;

#define ATSC3ESGSGDDTransportNodeAttribute_ipAddress                    0x0001
#define ATSC3ESGSGDDTransportNodeAttribute_port                         0x0002
#define ATSC3ESGSGDDTransportNodeAttribute_transmissionSessionId        0x0003

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t transportID;
    int32_t idDataOffset;
    int32_t version;
    int32_t fragmentEncoding;
    int32_t fragmentType;

    // The following is used by apps
    ENUserData userData;
    
    EnXMLParserDataOffsetArray fragments;
} ATSC3ESGSGDDFragmentNode;

#define ATSC3ESGSGDDFragmentNodeAttribute_transportID                   0x0001
#define ATSC3ESGSGDDFragmentNodeAttribute_id                            0x0002
#define ATSC3ESGSGDDFragmentNodeAttribute_version                       0x0004
#define ATSC3ESGSGDDFragmentNodeAttribute_fragmentEncoding              0x0008
#define ATSC3ESGSGDDFragmentNodeAttribute_fragmentType                  0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t contentLocationDataOffset;
    int32_t transportObjectID;
    
    EnXMLParserDataOffsetArray fragments;
} ATSC3ESGSGDDServiceGuideDeliveryUnitNode;

#define ATSC3ESGSGDDServiceGuideDeliveryUnitNodeAttribute_contentLocation 0x0001
#define ATSC3ESGSGDDServiceGuideDeliveryUnitNodeAttribute_transportObjectID 0x0002
#define ATSC3ESGSGDDServiceGuideDeliveryUnitChildNode_Fragment          0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    ATSC3ESGSGDDGroupingCriteriaNode groupingCriteria;
    ATSC3ESGSGDDTransportNode transport;
    ATSC3ESGSGDDServiceGuideDeliveryUnitNode serviceGuideDeliveryUnit;

} ATSC3ESGSGDDDescriptorEntryNode;

#define ATSC3ESGSGDDDescriptorEntryChildNode_GroupingCriteria           0x0001
#define ATSC3ESGSGDDDescriptorEntryChildNode_Transport                  0x0002
#define ATSC3ESGSGDDDescriptorEntryChildNode_ServiceGuideDeliveryUnit   0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t version;
    int32_t idDataOffset;
    EnXMLParserDataOffsetArray descriptorEntries;

    EnXMLParserDataBuffer dataBuffer;
} ATSC3ESGSGDDNode;

#define ATSC3ESGSGDDNodeAttribute_version                               0x0001
#define ATSC3ESGSGDDNodeAttribute_id                                    0x0002

typedef enum
{
    ATSC3ESGSGDDNodeType_Invalid = 0,
    ATSC3ESGSGDDNodeType_ServiceGuideDeliveryDescriptor = 1,
    ATSC3ESGSGDDNodeType_DescriptorEntry = 2,
    ATSC3ESGSGDDNodeType_GroupingCriteria = 3,
    ATSC3ESGSGDDNodeType_TimeGroupingCriteria = 4,
    ATSC3ESGSGDDNodeType_Transport = 5,
    ATSC3ESGSGDDNodeType_ServiceGuideDeliveryUnit = 6,
    ATSC3ESGSGDDNodeType_Fragment = 7,
} ATSC3ESGSGDDNodeType;

ENStatus
ATSC3ESGSGDDParse(const char *pXML, int32_t sXML,
    ATSC3ESGSGDDNode *pParsedSGDD);

ENStatus
ATSC3ESGSGDDFree(ATSC3ESGSGDDNode *pParsedSGDD);

#endif
