/*
 * Copyright © 2019-2021 Koherence, LLC (www.koherence.com).
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3FILEREADER_H
#define ENATSC3FILEREADER_H

typedef enum
{
    ENATSC3FileType_ALP = 1,
    ENATSC3FileType_Baseband = 2,
    ENATSC3FileType_ALPRaw = 3,
    ENATSC3FileType_J288ALP = 4,
    ENATSC3FileType_PCAPIPV4 = 5,
} ENATSC3FileType;

typedef void (*ENATSC3ALPLLSPLPsCallback)(uint64_t plpIdMask,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber,
    void *pUserData);

typedef void (*FileReaderALPPacketCallbackFunction)(uint32_t plpId,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber,
    int32_t fError, int32_t fDiscontinuity,
    const uint8_t *pPacket, int32_t sPacket, void *pUserData);

typedef void (*FileReaderIPV4PacketCallbackFunction)(uint32_t plpId,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber,
    const uint8_t *pPacket, int32_t sPacket, void *pUserData);

typedef void (*FileReaderBasebandPacketCallbackFunction)(uint32_t plpId,
    struct timespec *pPacketTimeUTC, uint64_t packetNumber,
    int32_t fError, int32_t fDiscontinuity, 
    const ENATCS3L1Time *pL1Time, const uint8_t *pPacket, 
    int32_t sPacket, void *pUserData);

int32_t
ENATSC3FileReaderReadALPFile(int fd, int32_t offsetSeconds, 
    FileReaderALPPacketCallbackFunction ALPPacketCallback,
    ENATSC3ALPLLSPLPsCallback LLSPLPsCallback, 
    int32_t *pFRunning, void *pUserData);

int32_t
ENATSC3FileReaderReadALPRawFile(int fd, 
    FileReaderALPPacketCallbackFunction ALPPacketCallback,
    ENATSC3ALPLLSPLPsCallback LLSPLPsCallback, int32_t *pFRunning, 
    void *pUserData);

int32_t
ENATSC3FileReaderReadBasebandFile(int fd, int32_t offsetSeconds, 
    FileReaderBasebandPacketCallbackFunction BasebandPacketCallback,
    ENATSC3ALPLLSPLPsCallback LLSPLPsCallback, int32_t *pFRunning, 
    void *pUserData);

int32_t
ENATSC3FileReaderReadJ288ALPFile(int fd, int32_t tOffsetSeconds,
    FileReaderALPPacketCallbackFunction ALPPacketCallback,
    ENATSC3ALPLLSPLPsCallback LLSPLPsCallback,
    int32_t *pFRunning, void *pUserData);

int32_t
ENATSC3FileReaderReadPCAPIPV4File(int fd, int32_t tOffsetSeconds,
    FileReaderIPV4PacketCallbackFunction IPV4PacketCallback,
    ENATSC3ALPLLSPLPsCallback LLSPLPsCallback,
    int32_t *pFRunning, void *pUserData);

#endif
