/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stddef.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <dirent.h>
#include <libgen.h>
#include <inttypes.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <arpa/inet.h>

#include "internetchecksum.h"

void
ENInternetChecksumInitialize(ENInternetChecksumContext *pContext)
{
    pContext->accum = 0xffff;
    pContext->fCarry = 0;
}

uint32_t
ENInternetChecksumFinalize(ENInternetChecksumContext *pContext)
{
    if (pContext->fCarry)
    {
        pContext->accum += (pContext->carry << 8);
        if (pContext->accum > 0xffff)
        {
            pContext->accum -= 0xffff;
        }
    }

    return ~pContext->accum & 0xffff;
}

void
ENInternetChecksumCalc(const uint8_t *pData, int32_t sData, 
    ENInternetChecksumContext *pContext)
{
    uint32_t accum;

    accum = pContext->accum;

    if (pContext->fCarry)
    {
        accum += (pContext->carry << 8) | *pData;  
        if (accum > 0xffff)
        {
            accum -= 0xffff;
        }
        pData++;
        sData--;
    }

    while (sData > 1)
    {
        accum += ntohs(*((uint16_t *)pData));
        if (accum > 0xffff)
        {
            accum -= 0xffff;
        }
        sData -= 2;
        pData += 2;
    }

    if (sData)
    {
        pContext->fCarry = 1;
        pContext->carry = *pData;
    }
    else
    {
        pContext->fCarry = 0;
    }

    pContext->accum = accum;
}


