/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3ROUTE_SLS_STSID_H
#define ENATSC3ROUTE_SLS_STSID_H

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t codePoint;
    uint32_t formatId;
    uint32_t frag;
    uint32_t order;
    uint32_t srcFecPayloadID;
} ENATSC3ROUTESLSSTSIDPayloadNode;

#define ENATSC3ROUTESLSSTSIDPayloadNodeAttribute_codePoint             0x0001
#define ENATSC3ROUTESLSSTSIDPayloadNodeAttribute_formatId              0x0002
#define ENATSC3ROUTESLSSTSIDPayloadNodeAttribute_frag                  0x0004
#define ENATSC3ROUTESLSSTSIDPayloadNodeAttribute_order                 0x0008
#define ENATSC3ROUTESLSSTSIDPayloadNodeAttribute_srcFecPayloadID       0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t schemeIdUriDataOffset;
    int32_t valueDataOffset;
} ENATSC3ROUTESLSSTSIDContentRatingNode;

#define ENATSC3ROUTESLSSTSIDContentRatingNodeAttribute_schemeIdUri     0x0001
#define ENATSC3ROUTESLSSTSIDContentRatingNodeAttribute_value           0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t startup;
    int32_t langDataOffset;
    int32_t repIdDataOffset;
    int32_t contentTypeDataOffset;

    ENATSC3ROUTESLSSTSIDContentRatingNode contentRating;
} ENATSC3ROUTESLSSTSIDMediaInfoNode;

#define ENATSC3ROUTESLSSTSIDMediaInfoNodeAttribute_startup             0x0001
#define ENATSC3ROUTESLSSTSIDMediaInfoNodeAttribute_lang                0x0002
#define ENATSC3ROUTESLSSTSIDMediaInfoNodeAttribute_repId               0x0004
#define ENATSC3ROUTESLSSTSIDMediaInfoNodeAttribute_contentType         0x0008
#define ENATSC3ROUTESLSSTSIDMediaInfoChildNode_ContentRating           0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t contentInfoDataOffset;

    ENATSC3ROUTESLSSTSIDMediaInfoNode mediaInfo;
} ENATSC3ROUTESLSSTSIDContentInfoNode;

#define ENATSC3ROUTESLSSTSIDContentInfoNodeAttribute_ContentInfo       0x0001
#define ENATSC3ROUTESLSSTSIDContentInfoChildNode_MediaInfo             0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t toi;
    uint64_t contentLength;
    uint64_t transferLength;
    int32_t contentLocationDataOffset;
    int32_t contentTypeDataOffset;
    int32_t contentEncodingDataOffset;
} ENATSC3ROUTESLSSTSIDFileNode;

#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_TOI                      0x0001
#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_ContentLocation          0x0002
#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_ContentType              0x0004
#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_ContentLength            0x0008
#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_TransferLength           0x0010
#define ENATSC3ROUTESLSSTSIDFileNodeAttribute_ContentEncoding          0x0020

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray files;           // Array of 
                                                // ATSC3SLSSTSIDFileNode
} ENATSC3ROUTESLSSTSIDFDTParametersNode;

#define ENATSC3ROUTESLSSTSIDFDTParametersChildNode_FDTFile             0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t fileTemplateDataOffset;

} ENATSC3ROUTESLSSTSIDFileTemplateNode;

#define ENATSC3ROUTESLSSTSIDFileTemplateNodeAttribute_fileTemplate     0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t efdtVersion;
    int64_t expiresSecondsUTC;
    int32_t fileTemplateDataOffset;

    EnXMLParserDataOffsetArray files;           // Array of
                                                // ATSC3SLSSTSIDFileNode
} ENATSC3ROUTESLSSTSIDFDTInstanceNode;

#define ENATSC3ROUTESLSSTSIDFDTInstanceNodeAttribute_efdtVersion       0x0001
#define ENATSC3ROUTESLSSTSIDFDTInstanceNodeAttribute_expires           0x0002
#define ENATSC3ROUTESLSSTSIDFDTInstanceNodeAttribute_fileTemplate      0x0004
#define ENATSC3ROUTESLSSTSIDFDTInstanceChildNode_File                  0x0008

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t version;

    ENATSC3ROUTESLSSTSIDFDTInstanceNode fdtInstance;
    ENATSC3ROUTESLSSTSIDFDTParametersNode fdtParameters;
    ENATSC3ROUTESLSSTSIDFileTemplateNode fileTemplate;
} ENATSC3ROUTESLSSTSIDEFDTNode;

#define ENATSC3ROUTESLSSTSIDEFDTNodeAttribute_version                  0x0001
#define ENATSC3ROUTESLSSTSIDEFDTChildNode_FDTInstance                  0x0002
// The following are not in spec but are used in the Sinclair CES test
// broadcast in place of FDTInstance
#define ENATSC3ROUTESLSSTSIDEFDTChildNode_FDTParameters                0x0004
#define ENATSC3ROUTESLSSTSIDEFDTChildNode_FileTemplate                 0x0008

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t rt;
    uint32_t minBuffSize;

    ENATSC3ROUTESLSSTSIDEFDTNode efdt;
    ENATSC3ROUTESLSSTSIDContentInfoNode contentInfo;
    EnXMLParserDataOffsetArray payloads;        // array of 
                                                // ENATSC3ROUTESLSSTSIDPayloadNode
} ENATSC3ROUTESLSSTSIDSrcFlowNode;

#define ENATSC3ROUTESLSSTSIDSrcFlowNodeAttribute_rt                    0x0001
#define ENATSC3ROUTESLSSTSIDSrcFlowNodeAttribute_minBuffSize           0x0002
#define ENATSC3ROUTESLSSTSIDSrcFlowChildNode_EFDT                      0x0004
#define ENATSC3ROUTESLSSTSIDSrcFlowChildNode_ContentInfo               0x0008
#define ENATSC3ROUTESLSSTSIDSrcFlowChildNode_Payload                   0x0010

typedef struct
{
    int32_t rt;
    EnXMLParserNodeBase base;
    uint32_t minBuffSize;

    ENATSC3ROUTESLSSTSIDEFDTNode efdt;
    ENATSC3ROUTESLSSTSIDContentInfoNode contentInfo;
    ENATSC3ROUTESLSSTSIDPayloadNode payload;
} ENATSC3ROUTESLSSTSIDRepairFlowNode;

#define ENATSC3ROUTESLSSTSIDRepairFlowNodeAttribute_rt                 0x0001
#define ENATSC3ROUTESLSSTSIDRepairFlowNodeAttribute_minBuffSize        0x0002
#define ENATSC3ROUTESLSSTSIDRepairFlowChildNode_EFDT                   0x0004
#define ENATSC3ROUTESLSSTSIDRepairFlowChildNode_ContentInfo            0x0008
#define ENATSC3ROUTESLSSTSIDRepairFlowChildNode_Payload                0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t tsi;
    uint32_t bw;
    int64_t startTimeSecondsUTC;
    int64_t endTimeSecondsUTC;
    ENATSC3ROUTESLSSTSIDSrcFlowNode srcFlow;
    ENATSC3ROUTESLSSTSIDRepairFlowNode repairFlow;
} ENATSC3ROUTESLSSTSIDLSNode;

#define ENATSC3ROUTESLSSTSIDLSNodeAttribute_tsi                        0x0001
#define ENATSC3ROUTESLSSTSIDLSNodeAttribute_bw                         0x0002
#define ENATSC3ROUTESLSSTSIDLSNodeAttribute_startTime                  0x0004
#define ENATSC3ROUTESLSSTSIDLSNodeAttribute_endTime                    0x0008
#define ENATSC3ROUTESLSSTSIDLSChildNode_SrcFlow                        0x0010
#define ENATSC3ROUTESLSSTSIDLSChildNode_RepairFlow                     0x0020

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t sIpAddrDataOffset;
    int32_t dIpAddrDataOffset;
    uint32_t dPort;

    EnXMLParserDataOffsetArray LSs;             // Array of ATSC3SLSSTSIDLSNode
} ENATSC3ROUTESLSSTSIDRSNode;

#define ENATSC3ROUTESLSSTSIDRSNodeAttribute_sIpAddr                    0x0001
#define ENATSC3ROUTESLSSTSIDRSNodeAttribute_dIpAddr                    0x0002
#define ENATSC3ROUTESLSSTSIDRSNodeAttribute_dPort                      0x0004
#define ENATSC3ROUTESLSSTSIDRSChildNode_LS                             0x0008

typedef struct _ENATSC3ROUTESLSSTSIDNode
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray RSs;             // Array of ATSC3SLSSTSIDRSNode

    EnXMLParserDataBuffer dataBuffer;
} ENATSC3ROUTESLSSTSIDNode, *ENATSC3ROUTESLSSTSIDNodePtr;

#define ENATSC3ROUTESLSSTSIDNodeAttribute_XMLNS                        0x0001
#define ENATSC3ROUTESLSSTSIDChildNode_RS                               0x0002

typedef enum
{
    ENATSC3ROUTESLSSTSIDNodeType_Invalid = 0,
    ENATSC3ROUTESLSSTSIDNodeType_STSID = 1,
    ENATSC3ROUTESLSSTSIDNodeType_RS = 2,
    ENATSC3ROUTESLSSTSIDNodeType_LS = 3,
    ENATSC3ROUTESLSSTSIDNodeType_SrcFlow = 4,
    ENATSC3ROUTESLSSTSIDNodeType_RepairFlow = 5,
    ENATSC3ROUTESLSSTSIDNodeType_EFDT = 6,
    ENATSC3ROUTESLSSTSIDNodeType_ContentInfo = 7,
    ENATSC3ROUTESLSSTSIDNodeType_Payload = 8,
    ENATSC3ROUTESLSSTSIDNodeType_File = 9,
    ENATSC3ROUTESLSSTSIDNodeType_FDTInstance = 10,
    ENATSC3ROUTESLSSTSIDNodeType_FileTemplate = 11,
    ENATSC3ROUTESLSSTSIDNodeType_FDTParameters = 12,
    ENATSC3ROUTESLSSTSIDNodeType_FDTFile = 13,
    ENATSC3ROUTESLSSTSIDNodeType_MediaInfo = 14,
    ENATSC3ROUTESLSSTSIDNodeType_ContentRating = 15,
} ENATSC3ROUTESLSSTSIDNodeType;

ENStatus
ENATSC3ROUTESLSSTSIDParse(const char *pXML, int32_t sXML,
    ENATSC3ROUTESLSSTSIDNode *pParsedSTSID);

ENStatus
ENATSC3ROUTESLSSTSIDFree(ENATSC3ROUTESLSSTSIDNode *pParsedSTSID);

#endif
