/*
 * Copyright © 2018-2019 Koherence, LLC (www.koherence.com). 
 * 
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of
 * the following permission added to Section 15 as permitted in Section 7(a):
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>

#include "libenutils_defs.h"
#include "libenutils.h"

static char vEncTable[] = 
{
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
    'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
    'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
    'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
    'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
    'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
    'w', 'x', 'y', 'z', '0', '1', '2', '3',
    '4', '5', '6', '7', '8', '9', '+', '/'
};

void
ENUtils_Base64Encode(const uint8_t *pData, int32_t nData, char *pOut, 
    int32_t *nOut)
{
    int32_t i;
    int32_t nTriples;
    int32_t nRemainder;
    uint32_t val;

    *nOut = 4 * ((nData + 2) / 3);

    nRemainder = nData % 3;
    nTriples = nData - nRemainder;
    for (i = 0; i < nTriples; i += 3) 
    {
        val = (pData[0] << 16) | (pData[1] << 8) | pData[2];
        pData += 3;

        pOut[0] = vEncTable[(val >> 18) & 0x3F];
        pOut[1] = vEncTable[(val >> 12) & 0x3F];
        pOut[2] = vEncTable[(val >> 6) & 0x3F];
        pOut[3] = vEncTable[(val >> 0) & 0x3F];
        pOut += 4;
    }

    if (nRemainder)
    {
        val = (pData[0] << 16);
        val |= (nRemainder > 1) ? (pData[1] << 8) : 0;

        pOut[0] = vEncTable[(val >> 18) & 0x3F];
        pOut[1] = vEncTable[(val >> 12) & 0x3F];
        pOut[2] = vEncTable[(val >> 6) & 0x3F];
        pOut[3] = vEncTable[(val >> 0) & 0x3F];

        switch (nRemainder)
        {
            case 1:
                pOut[3] = '=';
                pOut[2] = '=';
                break;
            case 2:
                pOut[3] = '=';
                break;
        }
    }
}

static uint8_t vDecTable[] = 
{
    0,   0,   0,   0,   0,   0,   0,   0, // 0 - 7 
    0,   0,   0,   0,   0,   0,   0,   0, // 8 - 15
    0,   0,   0,   0,   0,   0,   0,   0, // 16 - 23
    0,   0,   0,   0,   0,   0,   0,   0, // 24 - 31
    0,   0,   0,   0,   0,   0,   0,   0, // 32 - 39
    0,   0,   0,  62,   0,   0,   0,  63, // 40 - 47
   52,  53,  54,  55,  56,  57,  58,  59, // 48 - 55
   60,  61,   0,   0,   0,   0,   0,   0, // 56 - 63
    0,   0,   1,   2,   3,   4,   5,   6, // 64 - 71
    7,   8,   9,  10,  11,  12,  13,  14, // 72 - 79
   15,  16,  17,  18,  19,  20,  21,  22, // 80 - 87
   23,  24,  25,   0,   0,   0,   0,   0, // 88 - 95
    0,  26,  27,  28,  29,  30,  31,  32, // 96 - 103
   33,  34,  35,  36,  37,  38,  39,  40, // 104 - 111
   41,  42,  43,  44,  45,  46,  47,  48, // 112 - 117
   49,  50,  51,   0,   0,   0,   0,   9, // 120 - 127
    0,   0,   0,   0,   0,   0,   0,   0, // 128 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 136 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 144 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 152 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 160 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 168 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 176 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 184 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 192 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 200 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 208 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 216 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 224 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 232 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 240 - 135
    0,   0,   0,   0,   0,   0,   0,   0, // 248 - 255
};

int32_t
ENUtils_Base64Decode(const char *pData, int32_t nData, uint8_t *pOut, 
    int32_t *pNOut)
{
    int32_t nOut = 0;
    uint32_t val;
    int32_t rval = 0;

    if (nData % 4)
    {
        rval = 1;
        goto out;
    }
    
    while (nData > 4)
    {
        val = (vDecTable[(uint8_t)pData[0]] << 18) |
              (vDecTable[(uint8_t)pData[1]] << 12) |
              (vDecTable[(uint8_t)pData[2]] << 6) |
               vDecTable[(uint8_t)pData[3]];
        pOut[0] = (val >> 16) & 0xff;
        pOut[1] = (val >> 8) & 0xff;
        pOut[2] = val & 0xff;
        
        pData += 4;
        nData -= 4;
        pOut += 3;
        nOut += 3;
    }

    val = (vDecTable[(uint8_t)pData[0]] << 18) |
          (vDecTable[(uint8_t)pData[1]] << 12) |
          (vDecTable[(uint8_t)pData[2]] << 6) |
           vDecTable[(uint8_t)pData[3]];
    pOut[0] = (val >> 16) & 0xff;
    nOut++;
    if (pData[2] != '=')
    {
        pOut[1] = (val >> 8) & 0xff;
        nOut++;
        if (pData[3] != '=')
        {
            pOut[2] = val & 0xff;
            nOut++;
        }
    }

    *pNOut = nOut;

out:
    return rval;
}
